local uci = require "luci.model.uci".cursor()
local lfs = require "lfs"
require "muvirt"

function getAppStoreCachePath()
  cachepath = nil
  uci:foreach("appstore","appstore", function(s)
      name = s[".name"]
      if (s["listcache"] ~= nil and cachepath == nil) then
        cachepath = s["listcache"]
      end
    end)
    if (cachepath == nil) then
      cachepath = "/tmp/appstorelist/"
    end
  return cachepath
end

function getRepositoryFile(repository)
  local cachepath = getAppStoreCachePath()
  local f = io.open(cachepath .. repository)
  local s = f:read("*a")
  f:close()
  data = luci.jsonc.parse(s)
  return data
end

function assembleAppStoreApplianceList(cachepath)
  masterlist = {}
  for file in lfs.dir(cachepath) do
    if lfs.attributes(file, 'mode') ~='directory' then
      listdata = getRepositoryFile(file)
      if (listdata ~= nil and listdata["appliances"] ~= nil) then
      	masterlist[file] = listdata["appliances"]
      end
    end
  end
  return masterlist
end

function getApplianceInfo(repository, applianceID)
  local repoFile = getRepositoryFile(repository)
  if (repoFile == nil) then
    return nil
  end
  local repoAppliances = repoFile["appliances"]
  if (repoAppliances == nil) then
    return nil
  end
  for i,v in ipairs(repoAppliances) do
    if (v["id"] ~= nil and v["id"] == applianceID) then
      return v
    end
  end
  return nil
end

function deployAppliance(newSectionID, applianceSource, applianceID, configArguments)
  local returnData = {}
  local appliance = getApplianceInfo(applianceSource, applianceID)
  if (appliance == nil) then
    returnData["error"] = "appliance " .. applianceSource .. "/" .. applianceID .. " not found"
    return returnData
  end

  local newVMConfig = {}
  newVMConfig["imageurl"] = appliance["download"]
  newVMConfig["imagesum256"] = appliance["sha256sum"]

  if (appliance["format"] ~= nil) then
  	newVMConfig["imageformat"] = appliance["format"]
  end

  newVMConfig["provisioned"] = "0"
  newVMConfig["enable"] = "1"
  newVMConfig["mac"] = muvirt_generate_random_mac()
  newVMConfig["network"] = {muvirt_get_default_network()}

  if (appliance["mincpu"] ~= nil) then
    newVMConfig["numprocs"] = appliance["mincpu"]
  else
    newVMConfig["numprocs"] = "1"
  end

  if (appliance["minram"] ~= nil) then
    newVMConfig["memory"] = appliance["minram"]
  else
    newVMConfig["memory"] = "512"
  end

  if (appliance["mindisk"] ~= nil) then
    newVMConfig["disksize"] = appliance["mindisk"]
  else
    newVMConfig["disksize"] = "4096"
  end

  local saved = uci:section("virt","vm",newSectionID, newVMConfig)
  if (saved == false) then
    returnData["error"] = "appliance " .. newVMID .. " could not be saved, possibly due to invalid characters in VM ID"
    return returnData
  end

  uci:save("virt")
  uci.apply()

  newVMConfig["name"] = newSectionID
  return newVMConfig
end
